

/* ***********************************************************************
                Naval Air Warfare Center / Aircraft Division              
                        Patuxent Naval Air Station                         
 ========================================================================

        File:           parse_tdb.h                                         
                                                                        
        Classification: UNCLASSIFIED                                   
                                                                        
        Description:    This file contains the macros, structures,
                        and classes needed to extract and access
                        SWEG's sensors, platforms, and players, as
                        defined by the TDB

        Written by:     Alex Harper
*********************************************************************** */


#if !defined __PARSE_TDB_H
#define __PARSE_TDB_H

#include <string.h>

#include "antenna.h"


#define RADAR_RECEIVER          0x01
#define RADAR_TRANSMITTER       0x02

#define SENSOR_OTHER     0x00
#define SENSOR_EM        0x01
#define SENSOR_SONAR     0x02
#define SENSOR_IR        0x04
#define SENSOR_OPTICAL   0x05

#define FLAG_RECEIVER_LOSSES    0x01
#define FLAG_RECEIVER_NOISE     0x02
#define FLAG_RECEIVER_THRESHOLD 0x04
#define FLAG_ONE_M2             0x08
#define FLAG_ACQUISITION        0x10

class radar_type;
class platform_type;
class player_type;

/* ***********************************************************
   This class correlates platform names w/ semantic codes
*********************************************************** */
class name2sem : public dbl_llist {

   public:
      string_type name;          // platform name
      int semcode;               // platform semantic code
      
      virtual ~name2sem() {}

      name2sem *find(int x) {
         if (semcode == x) return this;
         if (next) return ((name2sem *)next)->find(x);
         return NULL;
      }
            
      name2sem *find(char *x) {
         if (!name.stringcmp(x)) return this;
         if (next) return ((name2sem *)next)->find(x);
         return NULL;
      }
            
};


/* ***********************************************************
  This class links transmitters w/ receivers
*********************************************************** */
class linkage_type : public dbl_llist {

   public:
      int first, second;                 // link values found under "LINKAGES" in the TDB
      radar_type *firstptr, *secondptr;  // pointers to systems represented by first/second
      
      linkage_type() { first = second = -1; firstptr = secondptr = NULL; }
      virtual ~linkage_type() { }
};


/* ***********************************************************
   This class represents either an interval of ranges or a single
   range
*********************************************************** */
class range_type : public dbl_llist {

   public:
      double range;     // range in meters
      vector2d minmax;  // an interval in meters
};


/* ***********************************************************
  This class represents radar xmitter/receiver capabilities
*********************************************************** */
class capability_type : public dbl_llist {

   public:
      string_type name;                     // name of capability
      unsigned int typeflag;                // system type - receiver/xmitter
      unsigned int sensor_type;             // ACOUSTIC/EM/IR

      unsigned int flags;                   // misc status flags

      table_type *transmission_loss_table;  // transmission_loss irregular table
      table_type *antenna_table;            // antenna irregular table
      double one_m2_detect_range;           // 1M squared detection value (M)
      double receiver_freq;                 // receiver frequency (HZ)
      double receiver_noise;                // receiver noise (DB)
      double receiver_loss;                 // receiver loss (DB)
      double operating_loss;                // operating loss (DB)
      double receiver_threshold;            // receiver threshold (DB)
      double transmitter_loss;              // transmitter loss (DB)
      double transmitter_power;             // transmitter power (DB)
      double duty_cycle;                    // duty cycle(no units)
      double pulse_ratio;                   // pulse ratio (DB)
      double prf;                           // pulse repetition freqency (HZ)
      double pulse_width;                   // pulse width (watts)
      vector2d minmax_el;                   // range of elevation (Radians)
      vector2d minmax_az;                   // range of azimuth (Radians)
      dbl_llist_manager range_list;         // list of ranges (alt-range)
      
      double presented_area_freq;           // IR signature (HZ)
      double noise_equiv_irradiance;        // IR reciever noise (DB)
      int    noise_equiv_irradiance_flag;   // flag if noise_equiv_irradiance is valid
      double pixel_fov;                     // pixel field of view for IR sensors (STERADIAN)
      
      capability_type();
      virtual ~capability_type();
      void extract_tdbdat(FILE *infile, char *token, int *errorflag);
      capability_type *find(char *x);
};


/* ***********************************************************
   This class is used to create a sub-linklist of capabilities from
   a larger linklist
*********************************************************** */
class capability_list_type : public dbl_llist {

   public:
      capability_type *item;		// capability in the master list
      
      capability_list_type() { item = NULL; }
      virtual ~capability_list_type() { }
};


/* ***********************************************************
   This class is used to encapsulate a list of parameters to various
   sensor functions
*********************************************************** */
class detection_data_type {

   public:
      double    rcs;              // cross section
      double    presented_area;   // frequency of the IR cross-section of the target
      vector3d  target_pos;       // target position
      vector3d  receiver_pos;     // receiver position
      vector3d  xmitter_pos;      // transmitter position
      double    atmos_atten;      // atmospheric attenuation
      double    beam_az;          // target to receiver azimuth
      double    beam_el;          // target to receiver elevation

      detection_data_type() {
         atmos_atten = beam_az = beam_el = 0.0;
         rcs = presented_area = 1.0;
      }

};


/* ***********************************************************
   This class represents a radar system
*********************************************************** */
class radar_type : public dbl_llist {

   protected:
      double calc_em_detection_range(detection_data_type *data);
      double calc_ir_detection_range(detection_data_type *data);

   public:

      string_type name;                 // name of radar system
      int id;                           // id used in "LINKAGES"
      dbl_llist_manager capability_man; // capabilities of this radar system
      linkage_type *links;

      name2sem *semcode;

      table_type *receiver_gain;        // receiver gain irregular table
      table_type *transmitter_gain;     // xmitter gain irregular table
      double transmitter_power;         // transmitter power (Watts)
      double wavelength2;               // wavelength (M)
      double fourpi_constant;           // 4PI constant for this radar (radians)
      double rxnoisedb;                 // Initial receiver noise (DB)
      double receiver_thresholddb;      // Detection threshold (DB)
      double total_lossdb;              // receiver+transmitter loss (DB)
      double receiver_freq;             // receiver frequency (HZ)
      double prf;                       // pulse repetition freqency (HZ)
      double pulse_width;                   // pulse width (watts)
      unsigned int sensor_type;         // SENSING/EM/ACOUSTIC
      double maxrange;                  // maximum detection range
            
      double presented_area_freq;           // IR signature (HZ)
      double noise_equiv_irradiance;        // IR reciever noise (DB)
      int    noise_equiv_irradiance_flag;   // flag if noise_equiv_irradiance is valid
      double pixel_fov;                     // pixel field of view for IR sensors (STERADIAN)
      
      radar_type();
      virtual ~radar_type() { }
      void extract_tdbdat(FILE *infile, char *token, int *errorflag, dbl_llist_manager *clist, int type, dbl_llist_manager *semcode_table);
      radar_type *find_radar(int x);

      void init_receiver(dbl_llist_manager *links);

      double calc_detection_range(detection_data_type *data);
      double calc_detection_range(antenna_orientation_type *target_state, dbl_llist_manager *em_reflect, double *receiver_pos, double *xmitter_pos, double *beam_az, double *beam_el);
      void dump_data(char *prefix);
      void data_cleanup();
      int query_justify();
};


/* ***********************************************************
   This class represent's a sweg platform
*********************************************************** */
class platform_type : public dbl_llist {

   public:
      string_type name;                 // name of platform
      dbl_llist_manager xmit, rec;      // xmitter/receiver system pointers
      dbl_llist_manager em_reflect;     // em reflect irregular tables
      player_type *parent;              // player/parent of platform

      name2sem *semcode;
            
      platform_type(player_type *p) { semcode = NULL; parent = p; }
      virtual ~platform_type() {}
      void extract_tdbdat(FILE *infile, char *token, int *errorflag, dbl_llist_manager *clist, dbl_llist_manager *susceptibility, dbl_llist_manager *semcode_table);
      radar_type *find_radar(int x);
      void dump_data(dbl_llist_manager *susceptibility, char *prefix);
      void data_cleanup();
      int query_justify();
};


/* ***********************************************************
   This class represent's a sweg player
*********************************************************** */
class player_type : public dbl_llist {

   public:
      string_type name;                 // name of player
      dbl_llist_manager plat_man;       // list of platforms in this player
      dbl_llist_manager linkage_man;    // system linkages

      virtual ~player_type() {}
   
      void extract_tdbdat(FILE *infile, char *token, int *errorflag, dbl_llist_manager *clist, dbl_llist_manager *susceptibility, dbl_llist_manager *semcode_table);
      void dump_data(dbl_llist_manager *susceptibility, char *prefix);
      void data_cleanup();
      int  query_justify();
      linkage_type *find_link(radar_type *radar);
};


/* ***********************************************************
   This class contains extracted TDB data about a scenario
*********************************************************** */
class TDB {

   protected:
      // set when input.dat is parsed
      int init_flag;
      
      int extract_dat(char *filename);
      int extract_capabilities(char *filename);
      int extract_players(char *filename);
      void dest();   

   public:
      dbl_llist_manager player_list;    // master list of players
      dbl_llist_manager clist;          // master list of capabilities
      dbl_llist_manager susceptibility; // master list of susceptibilities
      dbl_llist_manager semcode_table;  // semantic code correlation table
      
      TDB() { init_flag = 0; }
      
      virtual ~TDB() { dest(); }
      
      int extract_data(char *datname, char *lisname);
      void dump_data();
      void data_cleanup();
      
      platform_type *find_platform(int code);
};


int read_name2sem(char *filename, dbl_llist_manager *semcode_table);

#endif


