

/* ***********************************************************************
                Naval Air Warfare Center / Aircraft Division              
                        Patuxent Naval Air Station                         
 ========================================================================

        File:           tdb.c                                         
                                                                        
        Classification: UNCLASSIFIED                                   
                                                                        
        Description:    This file contains the functions, methods, and
                        global variables needed by the main interface
                        into the TDB.

        Written by:     Alex Harper
*********************************************************************** */


#include "dataio.h"
#include "global.h"

#include "parse_tdb.h"


/* ***********************************************************
  This method extracts data from a player block in the TDB
*********************************************************** */
int TDB::extract_dat(char *filename) {

   FILE *infile;
   int errorflag;
   char token[MAXSTRLEN];
   union {
      player_type *player;
      susceptibility_type *str;
      capability_type *ctr;
   };

   int flag = 0;
   
   if (!find_file(filename, "r", NULL, PLATFORM_SLASH, NULL, &infile))
      return 0;
      
   errorflag = get_sweg_token(infile, token);

   while (errorflag) {

      if (!flag) {
         if (!strcmp(token, TOKEN_TDB))
            flag = 1;

         errorflag = get_sweg_token(infile, token);
         continue;
      }

      if (!strcmp(token, TOKEN_END_TDB)) {
         errorflag = get_sweg_token(infile, token);
         flag = 0;
         break;
      }

      if (!strcmp(token, TOKEN_PLAYER_STRUCTURE)) {
         player_list.append(player = new player_type, NULL);
 
         player->extract_tdbdat(infile, token, &errorflag, &clist, &susceptibility, &semcode_table);      

         errorflag = get_sweg_token(infile, token);
         continue;
      }
      
      if (!strcmp(token, TOKEN_CAPABILITY)) {

         // capability name
         errorflag = get_sweg_token(infile, token);

         ctr = clist.head ? ((capability_type *)clist.head)->find(token) : (capability_type *)NULL;

         if (!ctr) {
            clist.insert(ctr = new capability_type, NULL);
            ctr->name.stringcpy(token);
         }

         // ctr has the current capability
         ctr->extract_tdbdat(infile, token, &errorflag);      
         continue;
      }
            
      /* *******************************
         parse susceptibility block 
      ******************************* */
      if (!strcmp(token, TOKEN_SUSCEPTIBILITY)) {

         // susceptibility name
         errorflag = get_sweg_token(infile, token);

         // make sure the susceptibility is in the list
         str = susceptibility.head ? ((susceptibility_type *)susceptibility.head)->find_table(token) : (susceptibility_type *)NULL;

         if (!str) {
            susceptibility.insert(str = new susceptibility_type, NULL);
            str->name.stringcpy(token);
         }
    
         // str has the current susceptibility
         str->extract_tdbdat(infile, token, &errorflag);      
         continue;
      }
            
      errorflag = get_sweg_token(infile, token);
   }

   fclose(infile);
   return 1;
}


/* ***********************************************************
  This method extracts data from the TDB
*********************************************************** */
int TDB::extract_data(char *datname, char *lisname) {

   dest();

   if (!find_file(datname, "r", NULL, PLATFORM_SLASH, NULL, NULL) ||
       !find_file(lisname, "r", NULL, PLATFORM_SLASH, NULL, NULL))
      return init_flag = 0;

   if (!read_name2sem(lisname, &semcode_table) || !extract_dat(datname)) {
      dest();
      return init_flag = 0;
   }

   return init_flag = 1;
}


/* ***********************************************************
  This method extracts data from the TDB
*********************************************************** */
void TDB::dest() {

   player_list.dest();
   clist.dest();
   susceptibility.dest();
   semcode_table.dest();
}

      
/* ***********************************************************
  This method dumps TDB data to stdout
*********************************************************** */
void TDB::dump_data() {

   player_type *ptr;
   
   for (ptr=(player_type *)player_list.head; ptr; ptr=(player_type *)ptr->next)
      ptr->dump_data(&susceptibility, "");
}


/* ***********************************************************
  This method eliminates useless data nodes after the extraction
  process
*********************************************************** */
void TDB::data_cleanup() {

   player_type *ptr, *qtr;
   susceptibility_type *ltr, *mtr;
   capability_type *ctr, *dtr;
   
   // clean up players  
   ptr = (player_type *)player_list.head;
   
   while (ptr) {
      qtr = ptr;
      ptr = (player_type *)ptr->next;
      
      qtr->data_cleanup();
      if (!qtr->query_justify()) {
         player_list.remove(qtr);
         delete qtr;
      }

   }

   // clean up susceptibilities                   
   ltr = (susceptibility_type *)susceptibility.head;
   
   while (ltr) {
      mtr = ltr;
      ltr = (susceptibility_type *)ltr->next;
      
      if (!mtr->rlist && !mtr->elist) {
         susceptibility.remove(mtr);
         delete mtr;
      }

   }

   // clean up capabilities                   
   ctr = (capability_type *)clist.head;
   
   while (ctr) {
      dtr = ctr;
      ctr = (capability_type *)ctr->next;

      if (!dtr->antenna_table) {
         clist.remove(dtr);
         delete dtr;
      }

   }

}


/* ***********************************************************
  This method searches for data on a specific platform type
*********************************************************** */
platform_type *TDB::find_platform(int code) {

   player_type *ptr;
   platform_type *plattr;
   
   if (!init_flag || !semcode_table.head)
      return NULL;

   for (ptr=(player_type *)player_list.head; ptr; ptr=(player_type *)ptr->next)
      for (plattr=(platform_type *)ptr->plat_man.head; plattr; plattr=(platform_type *)plattr->next)
         if (plattr->semcode && plattr->semcode->semcode == code)
            return plattr;

   return NULL;
}

